@@ -18,6 +18,13 @@
 #define __NR_sys_vmsplice	1302
 #endif
 
+#ifndef __NR_preadv2
+#define __NR_preadv2		1348
+#endif
+#ifndef __NR_pwritev2
+#define __NR_pwritev2		1349
+#endif
+
 #define nop		asm volatile ("hint @pause" ::: "memory");
 #define read_barrier()	asm volatile ("mf" ::: "memory")
 #define write_barrier()	asm volatile ("mf" ::: "memory")
@@ -34,9 +41,9 @@ static inline unsigned long arch_ffz(unsigned long bitmask)
 	return ia64_popcnt(bitmask & (~bitmask - 1));
 }
 
-static inline unsigned long get_cpu_clock(void)
+static inline unsigned long long get_cpu_clock(void)
 {
-	unsigned long ret;
+	unsigned long long ret;
 
 	__asm__ __volatile__("mov %0=ar.itc" : "=r" (ret) : : "memory");
 	return ret;
